/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "Surface.h"
#include "sge_draw.h"

#include <assert.h>

Surface::Surface() { sdlSurface = NULL; }

Surface::Surface(const int w, const int h, const Pixelformat &f) {
  sdlSurface = SDL_AllocSurface(SDL_SWSURFACE, w, h, f.bpp()*8, f.rMask(), f.gMask(), f.bMask(), f.aMask());
  
  if (sdlSurface != NULL)
    format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                             sdlSurface->format->Rmask,
			     sdlSurface->format->Gmask,
			     sdlSurface->format->Bmask,
			     sdlSurface->format->Amask);
  else
    format = NULL;
  
  maxCoord = Point(w - 1, h - 1);
}

Surface::Surface(const Surface& surf) {
  sdlSurface = SDL_AllocSurface(surf.sdlSurface->flags,
				 surf.sdlSurface->w,
				 surf.sdlSurface->h,
				 surf.sdlSurface->format->BitsPerPixel,
				 surf.sdlSurface->format->Rmask,
				 surf.sdlSurface->format->Gmask,
				 surf.sdlSurface->format->Bmask,
				 surf.sdlSurface->format->Amask);
  if (sdlSurface == NULL) {
    // FIXME: throw exceptin
    exit(2);
  }

  SDL_BlitSurface(surf.sdlSurface,0,sdlSurface,0);

  if (sdlSurface != NULL)
    format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                             sdlSurface->format->Rmask,
			     sdlSurface->format->Gmask,
			     sdlSurface->format->Bmask,
			     sdlSurface->format->Amask);
  else
    format = NULL;
  
  maxCoord = Point(sdlSurface->w - 1, sdlSurface->h - 1);
}

Surface::Surface(SDL_Surface *surf) {
  sdlSurface = surf;

  if (sdlSurface == NULL) {
    // FIXME: throw exceptin
    exit(2);
  }

  format = new Pixelformat(sdlSurface->format->BytesPerPixel,
                           sdlSurface->format->Rmask,
			   sdlSurface->format->Gmask,
			   sdlSurface->format->Bmask,
			   sdlSurface->format->Amask);
  
  maxCoord = Point(sdlSurface->w - 1, sdlSurface->h - 1);
}
 
Surface::~Surface() { 
  if (sdlSurface != NULL) SDL_FreeSurface(sdlSurface);
  if (format != NULL) delete format;
}

void Surface::blit(Surface &dst, const Point &p) const {
  sge_Blit(sdlSurface, dst.sdlSurface, 0, 0, p.getX(), p.getY(), width(), height());
}

void Surface::blit(Surface &dst, const Rect& srcArea, const Point &dstPoint) const {
  sge_Blit(sdlSurface, dst.sdlSurface, 
           srcArea.getUpperLeft().getX(), srcArea.getUpperLeft().getY(),
	   dstPoint.getX(), dstPoint.getY(), srcArea.width()+1, srcArea.height()+1);
}

void Surface::putpixel(const Point& p, const Color& c) {
  assert(sdlSurface != NULL);
  
  sge_PutPixel(sdlSurface, p.getX(), p.getY(), format->mapToPixel(c));
}  

Color Surface::getpixel(const Point &p) {
  return format->mapToColor(sge_GetPixel(sdlSurface, p.getX(), p.getY()));
}

void Surface::fill(const Color &color) {
  bar(Rect(Point(), maxCoord), color);
}

void Surface::bar(const Rect &r, const Color &c) {
  assert(sdlSurface != NULL);
  
  sge_FilledRect(sdlSurface, r.getUpperLeft().getX(), r.getUpperLeft().getY(),
                 r.getLowerRight().getX(), r.getLowerRight().getY(), 
		 format->mapToPixel(c));
}

void Surface::line(const Point &p1, const Point &p2, const Color &c) {
  assert(sdlSurface != NULL);

  sge_Line(sdlSurface, p1.getX(), p1.getY(), p2.getX(), p2.getY(), 
           format->mapToPixel(c));
}

void Surface::circle(const Point &p, const Uint16 r, const Color &c) {
  assert(sdlSurface != NULL);
  
  sge_Circle(sdlSurface, p.getX(), p.getY(), r, format->mapToPixel(c));
}

void Surface::filledCircle(const Point &p, const Uint16 r, const Color &c) {
  assert(sdlSurface != NULL);
  
  sge_FilledCircle(sdlSurface, p.getX(), p.getY(), r, format->mapToPixel(c));
}

void Surface::ellipse(const Point &p, const Uint16 rx, const Uint16 ry, const Color &c) {
  assert(sdlSurface != NULL);
  
  sge_Ellipse(sdlSurface, p.getX(), p.getY(), rx, ry, format->mapToPixel(c));
}

void Surface::filledEllipse(const Point &p, const Uint16 rx, const Uint16 ry, const Color &c) {
  assert(sdlSurface != NULL);
  
  sge_FilledEllipse(sdlSurface, p.getX(), p.getY(), rx, ry, format->mapToPixel(c));
}

void Surface::setClipRect(const Rect &r) {
  SDL_SetClipping(sdlSurface, r.getUpperLeft().getY(), r.getUpperLeft().getX(),
                  r.getLowerRight().getY(), r.getLowerRight().getX());
}

void Surface::clearClipRect() {
  SDL_SetClipping(sdlSurface, 0, 0, 0, 0);
}

void Surface::loadSprite(istream &in) {
  Uint16 w = (width()+7) >> 3;
  Uint8 *map = new Uint8[w * height()];
  
  // fill everything with transparent color
  fill(Color(0, 0, 0, 255));
  
  in.read(map, w * height());
  
  for (int y = 0; y < height(); y++)
    for (int x = 0; x < width(); x++)
      if (map[y * w + (x >> 3)] & (0x80 >> (x & 7)))
        putpixel(Point(x, y), Color(in));
	
  delete [] map;
}

void Surface::loadImage(istream &in) {
  for (int y = 0; y < height(); y++)
    for (int x = 0; x < width(); x++)
      putpixel(Point(x, y), Color(in));	
}

void Surface::text(const Point &p, const string &text, const Color &c, TTFont &f, const Sint8 vDir, const Sint8 hDir) {
  Surface *s = f.render(text, c);

  s->blit(*this, Point(p.getX() - ((hDir < 0) ? 0 : ((hDir == 0) ? s->width()/2 : s->width())),
                       p.getY() - ((vDir > 0) ? 0 : ((vDir == 0) ? s->height()/2 : s->height()))));
}